// Copyright 2015 The Chromium Authors. All rights reserved.
// Copyright (C) 2019 Apple Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//    * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//    * Neither the name of Google Inc. nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#pragma once

namespace WebCore {

enum TabSizeValueType {
    LengthValueType,
    SpaceValueType,
};

struct TabSize {
    TabSize(float numOrLength, TabSizeValueType isSpaces = SpaceValueType)
        : m_value(numOrLength)
        , m_isSpaces(isSpaces)
    {
    }

    bool isSpaces() const
    {
        return m_isSpaces;
    }

    float widthInPixels(float spaceWidth) const
    {
        return m_isSpaces ? m_value * spaceWidth : m_value;
    }

    float value() const
    {
        return m_value;
    }

    operator bool() const { return value(); }

    float m_value;
    bool m_isSpaces;
};

inline bool operator==(const TabSize& a, const TabSize& b)
{
    return (a.m_value == b.m_value) && (a.m_isSpaces == b.m_isSpaces);
}

inline bool operator!=(const TabSize& a, const TabSize& b)
{
    return !(a == b);
}

} // namespace WebCore
