/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "MessageNames.h"
#include <WebCore/Cookie.h>
#include <WebCore/MessageWithMessagePorts.h>
#include <WebCore/NetworkLoadInformation.h>
#include <WebCore/ResourceError.h>
#include <optional>
#include <wtf/Expected.h>
#include <wtf/Forward.h>
#include <wtf/Seconds.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebCore {
class NetworkLoadMetrics;
class ResourceError;
class ResourceResponse;
struct RequestStorageAccessResult;
}

namespace Messages {
namespace NetworkConnectionToWebProcess {

using PerformSynchronousLoadDelayedReply = CompletionHandler<void(const WebCore::ResourceError& error, const WebCore::ResourceResponse& response, const Vector<uint8_t>& data)>;

using TestProcessIncomingSyncMessagesWhenWaitingForSyncReplyDelayedReply = CompletionHandler<void(bool handled)>;

using SendH2PingAsyncReply = CompletionHandler<void(const Expected<Seconds, WebCore::ResourceError>& result)>;

using IsResourceLoadFinishedAsyncReply = CompletionHandler<void(bool isFinished)>;

using CookiesForDOMDelayedReply = CompletionHandler<void(const String& cookieString, bool didAccessSecureCookies)>;

using CookieRequestHeaderFieldValueDelayedReply = CompletionHandler<void(const String& cookieString, bool didAccessSecureCookies)>;

using GetRawCookiesDelayedReply = CompletionHandler<void(const Vector<WebCore::Cookie>& cookies)>;

using DeleteCookieAsyncReply = CompletionHandler<void()>;

using DomCookiesForHostDelayedReply = CompletionHandler<void(const Vector<WebCore::Cookie>& cookies)>;

using BlobSizeDelayedReply = CompletionHandler<void(uint64_t resultSize)>;

using WriteBlobsToTemporaryFilesForIndexedDBAsyncReply = CompletionHandler<void(const Vector<String>& fileNames)>;

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using ResourceLoadStatisticsUpdatedAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using HasStorageAccessAsyncReply = CompletionHandler<void(bool hasStorageAccess)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using RequestStorageAccessAsyncReply = CompletionHandler<void(const WebCore::RequestStorageAccessResult& result)>;
#endif

using GetNetworkLoadInformationResponseDelayedReply = CompletionHandler<void(const WebCore::ResourceResponse& response)>;

using GetNetworkLoadIntermediateInformationDelayedReply = CompletionHandler<void(const Vector<WebCore::NetworkTransactionInformation>& transactions)>;

using TakeNetworkLoadInformationMetricsDelayedReply = CompletionHandler<void(const WebCore::NetworkLoadMetrics& networkMetrics)>;

#if ENABLE(SERVICE_WORKER)
using EstablishSWContextConnectionAsyncReply = CompletionHandler<void()>;
#endif

using EstablishSharedWorkerContextConnectionAsyncReply = CompletionHandler<void()>;

using CreateRTCProviderAsyncReply = CompletionHandler<void()>;

#if ENABLE(WEB_RTC)
using ConnectToRTCDataChannelRemoteSourceAsyncReply = CompletionHandler<void(const std::optional<bool>& result)>;
#endif

using TakeAllMessagesForPortAsyncReply = CompletionHandler<void(const Vector<WebCore::MessageWithMessagePorts>& messages, uint64_t messageBatchIdentifier)>;

using CheckRemotePortForActivityAsyncReply = CompletionHandler<void(bool hasActivity)>;

#if PLATFORM(MAC)
using GetProcessDisplayNameAsyncReply = CompletionHandler<void(const String& displayName)>;
#endif

} // namespace NetworkConnectionToWebProcess
} // namespace Messages
