/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "Attachment.h"
#include "DataReference.h"
#include "DataTaskIdentifier.h"
#include "MessageNames.h"
#include "WebPushMessage.h"
#include "WebResourceLoadStatisticsStore.h"
#include <WebCore/SecurityOriginData.h>
#include <optional>
#include <wtf/Forward.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebCore {
enum class HTTPCookieAcceptPolicy : uint8_t;
}

namespace WebKit {
struct AppPrivacyReportTestingData;
struct WebsiteData;
}

namespace Messages {
namespace NetworkProcess {

using InitializeNetworkProcessAsyncReply = CompletionHandler<void()>;

using CreateNetworkConnectionToWebProcessAsyncReply = CompletionHandler<void(const std::optional<IPC::Attachment>& connectionIdentifier, WebCore::HTTPCookieAcceptPolicy cookieAcceptPolicy)>;

using FetchWebsiteDataAsyncReply = CompletionHandler<void(const WebKit::WebsiteData& websiteData)>;

using DeleteWebsiteDataAsyncReply = CompletionHandler<void()>;

using DeleteWebsiteDataForOriginsAsyncReply = CompletionHandler<void()>;

using RenameOriginInWebsiteDataAsyncReply = CompletionHandler<void()>;

using WebsiteDataOriginDirectoryForTestingAsyncReply = CompletionHandler<void(const String& directory)>;

using CancelDownloadAsyncReply = CompletionHandler<void(const IPC::DataReference& resumeData)>;

using DataTaskWithRequestAsyncReply = CompletionHandler<void(const WebKit::DataTaskIdentifier& taskIdentifier)>;

using SetAllowsAnySSLCertificateForWebSocketDelayedReply = CompletionHandler<void()>;

using FlushCookiesAsyncReply = CompletionHandler<void()>;

using SetCacheModelSynchronouslyForTestingDelayedReply = CompletionHandler<void()>;

using ProcessWillSuspendImminentlyForTestingSyncDelayedReply = CompletionHandler<void()>;

using PrepareToSuspendAsyncReply = CompletionHandler<void()>;

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using ClearPrevalentResourceAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using ClearUserInteractionAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using DumpResourceLoadStatisticsAsyncReply = CompletionHandler<void(const String& dumpedStatistics)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using UpdatePrevalentDomainsToBlockCookiesForAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using IsGrandfatheredAsyncReply = CompletionHandler<void(bool isGrandfathered)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using IsPrevalentResourceAsyncReply = CompletionHandler<void(bool isPrevalent)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using IsVeryPrevalentResourceAsyncReply = CompletionHandler<void(bool isVeryPrevalent)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetLastSeenAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using MergeStatisticForTestingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using InsertExpiredStatisticForTestingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetPrevalentResourceAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetPrevalentResourceForDebugModeAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using IsResourceLoadStatisticsEphemeralAsyncReply = CompletionHandler<void(bool isEphemeral)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using HadUserInteractionAsyncReply = CompletionHandler<void(bool hadUserInteraction)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using IsRelationshipOnlyInDatabaseOnceAsyncReply = CompletionHandler<void(bool hadUserInteraction)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using HasLocalStorageAsyncReply = CompletionHandler<void(bool hadUserInteraction)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using GetAllStorageAccessEntriesAsyncReply = CompletionHandler<void(const Vector<String>& domains)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using IsRegisteredAsRedirectingToAsyncReply = CompletionHandler<void(bool isRedirectingTo)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using IsRegisteredAsSubFrameUnderAsyncReply = CompletionHandler<void(bool isSubframeUnder)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using IsRegisteredAsSubresourceUnderAsyncReply = CompletionHandler<void(bool isSubresourceUnder)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using DomainIDExistsInDatabaseAsyncReply = CompletionHandler<void(bool exists)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using LogUserInteractionAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using ResetParametersToDefaultValuesAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using ScheduleClearInMemoryAndPersistentAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using ScheduleCookieBlockingUpdateAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using ScheduleStatisticsAndDataRecordsProcessingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using StatisticsDatabaseHasAllTablesAsyncReply = CompletionHandler<void(bool hasAllTables)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetCacheMaxAgeCapForPrevalentResourcesAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetGrandfatheredAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using GetResourceLoadStatisticsDataSummaryAsyncReply = CompletionHandler<void(const Vector<WebKit::WebResourceLoadStatisticsStore::ThirdPartyData>& thirdPartyData)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetGrandfatheringTimeAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetMaxStatisticsEntriesAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetMinimumTimeBetweenDataRecordsRemovalAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetPruneEntriesDownToAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetShouldClassifyResourcesBeforeDataRecordsRemovalAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetNotifyPagesWhenDataRecordsWereScannedAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetResourceLoadStatisticsTimeAdvanceForTestingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetIsRunningResourceLoadStatisticsTestAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetResourceLoadStatisticsDebugModeAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetVeryPrevalentResourceAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetSubframeUnderTopFrameDomainAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetSubresourceUnderTopFrameDomainAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetSubresourceUniqueRedirectToAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetSubresourceUniqueRedirectFromAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetTimeToLiveUserInteractionAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetTopFrameUniqueRedirectToAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetTopFrameUniqueRedirectFromAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using ResetCacheMaxAgeCapForPrevalentResourcesAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetCrossSiteLoadWithLinkDecorationForTestingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using ResetCrossSiteLoadsWithLinkDecorationForTestingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using DeleteCookiesForTestingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using HasIsolatedSessionAsyncReply = CompletionHandler<void(bool hasIsolatedSession)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using CloseITPDatabaseAsyncReply = CompletionHandler<void()>;
#endif

#if (ENABLE(INTELLIGENT_TRACKING_PREVENTION) && ENABLE(APP_BOUND_DOMAINS))
using SetAppBoundDomainsForResourceLoadStatisticsAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetShouldDowngradeReferrerForTestingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetThirdPartyCookieBlockingModeAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetShouldEnbleSameSiteStrictEnforcementForTestingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetFirstPartyWebsiteDataRemovalModeForTestingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetToSameSiteStrictCookiesForTestingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetFirstPartyHostCNAMEDomainForTestingAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetThirdPartyCNAMEDomainForTestingAsyncReply = CompletionHandler<void()>;
#endif

using SyncLocalStorageAsyncReply = CompletionHandler<void()>;

using DumpPrivateClickMeasurementAsyncReply = CompletionHandler<void(const String& privateClickMeasurementState)>;

using ClearPrivateClickMeasurementAsyncReply = CompletionHandler<void()>;

using SetPrivateClickMeasurementOverrideTimerForTestingAsyncReply = CompletionHandler<void()>;

using MarkAttributedPrivateClickMeasurementsAsExpiredForTestingAsyncReply = CompletionHandler<void()>;

using SetPrivateClickMeasurementEphemeralMeasurementForTestingAsyncReply = CompletionHandler<void()>;

using SimulatePrivateClickMeasurementSessionRestartAsyncReply = CompletionHandler<void()>;

using SetPrivateClickMeasurementTokenPublicKeyURLForTestingAsyncReply = CompletionHandler<void()>;

using SetPrivateClickMeasurementTokenSignatureURLForTestingAsyncReply = CompletionHandler<void()>;

using SetPrivateClickMeasurementAttributionReportURLsForTestingAsyncReply = CompletionHandler<void()>;

using MarkPrivateClickMeasurementsAsExpiredForTestingAsyncReply = CompletionHandler<void()>;

using SetPCMFraudPreventionValuesForTestingAsyncReply = CompletionHandler<void()>;

using SetPrivateClickMeasurementAppBundleIDForTestingAsyncReply = CompletionHandler<void()>;

using ClosePCMDatabaseAsyncReply = CompletionHandler<void()>;

using SetServiceWorkerFetchTimeoutForTestingDelayedReply = CompletionHandler<void()>;

using ResetServiceWorkerFetchTimeoutForTestingDelayedReply = CompletionHandler<void()>;

using ResetQuotaAsyncReply = CompletionHandler<void()>;

#if PLATFORM(IOS_FAMILY)
using SetBackupExclusionPeriodForTestingAsyncReply = CompletionHandler<void()>;
#endif

using ClearStorageAsyncReply = CompletionHandler<void()>;

#if ENABLE(APP_BOUND_DOMAINS)
using HasAppBoundSessionAsyncReply = CompletionHandler<void(bool hasAppBoundSession)>;
#endif

#if ENABLE(APP_BOUND_DOMAINS)
using ClearAppBoundSessionAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY) && !PLATFORM(MACCATALYST)
using ClearServiceWorkerEntitlementOverrideAsyncReply = CompletionHandler<void()>;
#endif

using UpdateBundleIdentifierAsyncReply = CompletionHandler<void()>;

using ClearBundleIdentifierAsyncReply = CompletionHandler<void()>;

#if PLATFORM(COCOA)
using AppPrivacyReportTestingDataAsyncReply = CompletionHandler<void(const WebKit::AppPrivacyReportTestingData& data)>;
#endif

#if PLATFORM(COCOA)
using ClearAppPrivacyReportTestingDataAsyncReply = CompletionHandler<void()>;
#endif

using CountNonDefaultSessionSetsAsyncReply = CompletionHandler<void(size_t count)>;

#if ENABLE(SERVICE_WORKER)
using GetPendingPushMessagesAsyncReply = CompletionHandler<void(const Vector<WebKit::WebPushMessage>& messages)>;
#endif

#if ENABLE(SERVICE_WORKER)
using ProcessPushMessageAsyncReply = CompletionHandler<void(bool didSucceed)>;
#endif

#if ENABLE(SERVICE_WORKER)
using ProcessNotificationEventAsyncReply = CompletionHandler<void(bool didSucceed)>;
#endif

using SetPushAndNotificationsEnabledForOriginAsyncReply = CompletionHandler<void()>;

using DeletePushAndNotificationRegistrationAsyncReply = CompletionHandler<void(const String& errorMessage)>;

using GetOriginsWithPushAndNotificationPermissionsAsyncReply = CompletionHandler<void(const Vector<WebCore::SecurityOriginData>& origins)>;

using GetOriginsWithPushSubscriptionsAsyncReply = CompletionHandler<void(const Vector<WebCore::SecurityOriginData>& origins)>;

using HasPushSubscriptionForTestingAsyncReply = CompletionHandler<void(bool hasSubscription)>;

} // namespace NetworkProcess
} // namespace Messages
