/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "MessageNames.h"
#include <WebCore/PageIdentifier.h>
#include <WebCore/RegistrableDomain.h>
#include <optional>
#include <wtf/Forward.h>
#include <wtf/HashSet.h>
#include <wtf/text/WTFString.h>

namespace WebCore {
class Credential;
}

namespace WebKit {
enum class AuthenticationChallengeDisposition : uint8_t;
}

namespace Messages {
namespace NetworkProcessProxy {

using TestProcessIncomingSyncMessagesWhenWaitingForSyncReplyDelayedReply = CompletionHandler<void(bool handled)>;

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using RequestStorageAccessConfirmAsyncReply = CompletionHandler<void(bool userDidGrantAccess)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using DeleteWebsiteDataInUIProcessForRegistrableDomainsAsyncReply = CompletionHandler<void(const HashSet<WebCore::RegistrableDomain>& domainsWithMatchingDataRecords)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using SetDomainsWithCrossPageStorageAccessAsyncReply = CompletionHandler<void()>;
#endif

using EstablishRemoteWorkerContextConnectionToNetworkProcessAsyncReply = CompletionHandler<void()>;

#if ENABLE(APP_BOUND_DOMAINS)
using GetAppBoundDomainsAsyncReply = CompletionHandler<void(const HashSet<WebCore::RegistrableDomain>& appBoundDomains)>;
#endif

using RequestStorageSpaceAsyncReply = CompletionHandler<void(const std::optional<uint64_t>& newQuota)>;

using TriggerBrowsingContextGroupSwitchForNavigationAsyncReply = CompletionHandler<void(bool success)>;

#if ENABLE(APPLE_PAY_REMOTE_UI_USES_SCENE)
using GetWindowSceneIdentifierForPaymentPresentationAsyncReply = CompletionHandler<void(const String& sceneID)>;
#endif

using DataTaskReceivedChallengeAsyncReply = CompletionHandler<void(WebKit::AuthenticationChallengeDisposition disposition, const WebCore::Credential& credential)>;

using DataTaskWillPerformHTTPRedirectionAsyncReply = CompletionHandler<void(bool allowed)>;

using DataTaskDidReceiveResponseAsyncReply = CompletionHandler<void(bool allowed)>;

using OpenWindowFromServiceWorkerAsyncReply = CompletionHandler<void(const std::optional<WebCore::PageIdentifier>& newPage)>;

using NavigateServiceWorkerClientAsyncReply = CompletionHandler<void(const std::optional<WebCore::PageIdentifier>& page)>;

} // namespace NetworkProcessProxy
} // namespace Messages
