/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "FileSystemStorageError.h"
#include "MessageNames.h"
#include "SharedFileHandle.h"
#include "StorageAreaIdentifier.h"
#include <WebCore/FileSystemHandleIdentifier.h>
#include <WebCore/FileSystemSyncAccessHandleIdentifier.h>
#include <optional>
#include <utility>
#include <wtf/Expected.h>
#include <wtf/Forward.h>
#include <wtf/HashMap.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>


namespace Messages {
namespace NetworkStorageManager {

using PersistedAsyncReply = CompletionHandler<void(bool persisted)>;

using PersistAsyncReply = CompletionHandler<void(bool persisted)>;

using FileSystemGetDirectoryAsyncReply = CompletionHandler<void(const Expected<WebCore::FileSystemHandleIdentifier, WebKit::FileSystemStorageError>& result)>;

using IsSameEntryAsyncReply = CompletionHandler<void(bool result)>;

using GetFileHandleAsyncReply = CompletionHandler<void(const Expected<WebCore::FileSystemHandleIdentifier, WebKit::FileSystemStorageError>& result)>;

using GetDirectoryHandleAsyncReply = CompletionHandler<void(const Expected<WebCore::FileSystemHandleIdentifier, WebKit::FileSystemStorageError>& result)>;

using RemoveEntryAsyncReply = CompletionHandler<void(const std::optional<WebKit::FileSystemStorageError>& result)>;

using ResolveAsyncReply = CompletionHandler<void(const Expected<Vector<String>, WebKit::FileSystemStorageError>& result)>;

using MoveAsyncReply = CompletionHandler<void(const std::optional<WebKit::FileSystemStorageError>& result)>;

using GetFileAsyncReply = CompletionHandler<void(const Expected<String, WebKit::FileSystemStorageError>& result)>;

using CreateSyncAccessHandleAsyncReply = CompletionHandler<void(const Expected<std::pair<WebCore::FileSystemSyncAccessHandleIdentifier, IPC::SharedFileHandle>, WebKit::FileSystemStorageError>& result)>;

using CloseSyncAccessHandleAsyncReply = CompletionHandler<void(const std::optional<WebKit::FileSystemStorageError>& result)>;

using GetHandleNamesAsyncReply = CompletionHandler<void(const Expected<Vector<String>, WebKit::FileSystemStorageError>& result)>;

using GetHandleAsyncReply = CompletionHandler<void(const Expected<std::pair<WebCore::FileSystemHandleIdentifier, bool>, WebKit::FileSystemStorageError>& result)>;

using ConnectToStorageAreaAsyncReply = CompletionHandler<void(const WebKit::StorageAreaIdentifier& identifier, const HashMap<String, String>& items, uint64_t messageIdentifier)>;

using ConnectToStorageAreaSyncDelayedReply = CompletionHandler<void(const WebKit::StorageAreaIdentifier& identifier, const HashMap<String, String>& items, uint64_t messageIdentifier)>;

using SetItemAsyncReply = CompletionHandler<void(bool hasError, const HashMap<String, String>& allItems)>;

using RemoveItemAsyncReply = CompletionHandler<void()>;

using ClearAsyncReply = CompletionHandler<void()>;

} // namespace NetworkStorageManager
} // namespace Messages
