/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "DataReference.h"
#include "EditorState.h"
#include "FocusedElementInformation.h"
#if PLATFORM(IOS_FAMILY)
#include "GestureTypes.h"
#endif
#include "MessageNames.h"
#include "ShareableBitmap.h"
#include "SharedBufferReference.h"
#include "SharedMemory.h"
#include "WebFoundTextRange.h"
#include <WebCore/ApplicationManifest.h>
#include <WebCore/DictationContext.h>
#include <WebCore/ElementContext.h>
#include <WebCore/FloatRect.h>
#include <WebCore/GraphicsLayer.h>
#include <WebCore/IntRect.h>
#include <WebCore/JSDOMExceptionHandling.h>
#include <WebCore/RectEdges.h>
#include <WebCore/RegistrableDomain.h>
#include <WebCore/SharedBuffer.h>
#include <WebCore/TextIndicator.h>
#include <WebCore/TextManipulationController.h>
#include <optional>
#include <wtf/Forward.h>
#include <wtf/OptionSet.h>
#include <wtf/RefCounted.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace IPC {
class SharedBufferReference;
}

namespace WebCore {
class FloatRect;
class FloatSize;
class IntPoint;
class IntRect;
class ResourceError;
class ResourceRequest;
struct AttributedString;
struct FontAttributes;
}

namespace WebKit {
class RevealItem;
enum class GestureRecognizerState : uint8_t;
enum class GestureType : uint8_t;
enum class MediaPlaybackState : uint8_t;
enum class SelectionTouch : uint8_t;
enum class TextRecognitionUpdateResult : uint8_t;
struct DataDetectionResult;
struct DocumentEditingContext;
struct EditingRange;
struct FrameInfoData;
struct FrameTreeNodeData;
struct UserMessage;
struct WebAutocorrectionData;
}

namespace Messages {
namespace WebPage {

using SetInitialFocusAsyncReply = CompletionHandler<void()>;

using SetActivityStateAsyncReply = CompletionHandler<void()>;

using TestProcessIncomingSyncMessagesWhenWaitingForSyncReplyDelayedReply = CompletionHandler<void(bool handled)>;

using ExecuteEditCommandWithCallbackAsyncReply = CompletionHandler<void()>;

#if PLATFORM(IOS_FAMILY)
using SelectWithGestureAsyncReply = CompletionHandler<void(const WebCore::IntPoint& point, WebKit::GestureType gestureType, WebKit::GestureRecognizerState gestureState, const OptionSet<WebKit::SelectionFlags>& flags)>;
#endif

#if PLATFORM(IOS_FAMILY)
using UpdateSelectionWithTouchesAsyncReply = CompletionHandler<void(const WebCore::IntPoint& point, WebKit::SelectionTouch touch, const OptionSet<WebKit::SelectionFlags>& selectionFlags)>;
#endif

#if PLATFORM(IOS_FAMILY)
using SelectWithTwoTouchesAsyncReply = CompletionHandler<void(const WebCore::IntPoint& point, WebKit::GestureType gestureType, WebKit::GestureRecognizerState gestureState, const OptionSet<WebKit::SelectionFlags>& selectionFlags)>;
#endif

#if PLATFORM(IOS_FAMILY)
using ExtendSelectionAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using ExtendSelectionForReplacementAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using MoveSelectionByOffsetAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using SelectTextWithGranularityAtPointAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using SelectPositionAtBoundaryWithDirectionAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using MoveSelectionAtBoundaryWithDirectionAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using SelectPositionAtPointAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using BeginSelectionInDirectionAsyncReply = CompletionHandler<void(bool endIsMoving)>;
#endif

#if PLATFORM(IOS_FAMILY)
using UpdateSelectionWithExtentPointAsyncReply = CompletionHandler<void(bool endIsMoving)>;
#endif

#if PLATFORM(IOS_FAMILY)
using UpdateSelectionWithExtentPointAndBoundaryAsyncReply = CompletionHandler<void(bool endIsMoving)>;
#endif

#if PLATFORM(IOS_FAMILY)
using RequestDictationContextAsyncReply = CompletionHandler<void(const String& selectedText, const String& textBefore, const String& textAfter)>;
#endif

#if (PLATFORM(IOS_FAMILY) && ENABLE(REVEAL))
using RequestRVItemInCurrentSelectedRangeAsyncReply = CompletionHandler<void(const WebKit::RevealItem& item)>;
#endif

#if (PLATFORM(IOS_FAMILY) && ENABLE(REVEAL))
using PrepareSelectionForContextMenuWithLocationInViewAsyncReply = CompletionHandler<void(bool shouldShowMenu, const WebKit::RevealItem& item)>;
#endif

#if PLATFORM(IOS_FAMILY)
using RequestAutocorrectionDataAsyncReply = CompletionHandler<void(const WebKit::WebAutocorrectionData& data)>;
#endif

#if PLATFORM(IOS_FAMILY)
using ApplyAutocorrectionAsyncReply = CompletionHandler<void(const String& string)>;
#endif

#if PLATFORM(IOS_FAMILY)
using SyncApplyAutocorrectionDelayedReply = CompletionHandler<void(bool autocorrectionApplied)>;
#endif

#if PLATFORM(IOS_FAMILY)
using RequestEvasionRectsAboveSelectionAsyncReply = CompletionHandler<void(const Vector<WebCore::FloatRect>& rects)>;
#endif

#if PLATFORM(IOS_FAMILY)
using PerformActionOnElementAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using FocusNextFocusedElementAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using GetSelectionContextAsyncReply = CompletionHandler<void(const String& selectedText, const String& textBefore, const String& textAfter)>;
#endif

#if PLATFORM(IOS_FAMILY)
using GetRectsForGranularityWithSelectionOffsetAsyncReply = CompletionHandler<void(const Vector<WebCore::SelectionGeometry>& rect)>;
#endif

#if PLATFORM(IOS_FAMILY)
using GetRectsAtSelectionOffsetWithTextAsyncReply = CompletionHandler<void(const Vector<WebCore::SelectionGeometry>& rect)>;
#endif

#if PLATFORM(IOS_FAMILY)
using RequestFocusedElementInformationAsyncReply = CompletionHandler<void(const std::optional<WebKit::FocusedElementInformation>& info)>;
#endif

#if PLATFORM(IOS_FAMILY)
using UpdateSelectionWithDeltaAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using RequestDocumentEditingContextAsyncReply = CompletionHandler<void(const WebKit::DocumentEditingContext& response)>;
#endif

#if PLATFORM(IOS_FAMILY)
using InsertTextPlaceholderAsyncReply = CompletionHandler<void(const std::optional<WebCore::ElementContext>& placeholder)>;
#endif

#if PLATFORM(IOS_FAMILY)
using RemoveTextPlaceholderAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using TextInputContextsInRectAsyncReply = CompletionHandler<void(const Vector<struct WebCore::ElementContext>& contexts)>;
#endif

#if PLATFORM(IOS_FAMILY)
using FocusTextInputContextAndPlaceCaretAsyncReply = CompletionHandler<void(bool success)>;
#endif

#if PLATFORM(IOS_FAMILY)
using ClearServiceWorkerEntitlementOverrideAsyncReply = CompletionHandler<void()>;
#endif

using RequestImageBitmapAsyncReply = CompletionHandler<void(const WebKit::ShareableBitmap::Handle& image, const String& sourceMIMEType)>;

using GetPDFFirstPageSizeAsyncReply = CompletionHandler<void(const WebCore::FloatSize& size)>;

using GetContentsAsStringAsyncReply = CompletionHandler<void(const String& string)>;

using GetAllFramesAsyncReply = CompletionHandler<void(const WebKit::FrameTreeNodeData& mainFrame)>;

#if PLATFORM(COCOA)
using GetContentsAsAttributedStringAsyncReply = CompletionHandler<void(const WebCore::AttributedString& result)>;
#endif

#if ENABLE(MHTML)
using GetContentsAsMHTMLDataAsyncReply = CompletionHandler<void(const IPC::SharedBufferReference& data)>;
#endif

using GetMainResourceDataOfFrameAsyncReply = CompletionHandler<void(const std::optional<IPC::SharedBufferReference>& buffer)>;

using GetResourceDataFromFrameAsyncReply = CompletionHandler<void(const std::optional<IPC::SharedBufferReference>& buffer)>;

using GetRenderTreeExternalRepresentationAsyncReply = CompletionHandler<void(const String& string)>;

using GetSelectionOrContentsAsStringAsyncReply = CompletionHandler<void(const String& string)>;

using GetSelectionAsWebArchiveDataAsyncReply = CompletionHandler<void(const std::optional<IPC::SharedBufferReference>& data)>;

using GetSourceForFrameAsyncReply = CompletionHandler<void(const String& string)>;

using GetWebArchiveOfFrameAsyncReply = CompletionHandler<void(const std::optional<IPC::SharedBufferReference>& dataReference)>;

using RunJavaScriptInFrameInScriptWorldAsyncReply = CompletionHandler<void(const IPC::DataReference& resultData, const std::optional<WebCore::ExceptionDetails>& details)>;

using GetAccessibilityTreeDataAsyncReply = CompletionHandler<void(const std::optional<IPC::SharedBufferReference>& dataReference)>;

using ForceRepaintAsyncReply = CompletionHandler<void()>;

#if ENABLE(DATA_DETECTION)
using DetectDataInAllFramesAsyncReply = CompletionHandler<void(const WebKit::DataDetectionResult& result)>;
#endif

#if ENABLE(DATA_DETECTION)
using RemoveDataDetectedLinksAsyncReply = CompletionHandler<void(const WebKit::DataDetectionResult& result)>;
#endif

using SuspendAsyncReply = CompletionHandler<void(bool success)>;

using ResumeAsyncReply = CompletionHandler<void(bool success)>;

using TryCloseAsyncReply = CompletionHandler<void(bool shouldClose)>;

using ValidateCommandAsyncReply = CompletionHandler<void(bool isEnabled, int32_t state)>;

using RequestFontAttributesAtSelectionStartAsyncReply = CompletionHandler<void(const WebCore::FontAttributes& attributes)>;

using FindStringAsyncReply = CompletionHandler<void(bool found)>;

using ReplaceMatchesAsyncReply = CompletionHandler<void(uint64_t numberOfReplacements)>;

using FindRectsForStringMatchesAsyncReply = CompletionHandler<void(const Vector<WebCore::FloatRect>& matches)>;

using FindTextRangesForStringMatchesAsyncReply = CompletionHandler<void(const Vector<WebKit::WebFoundTextRange>& ranges)>;

using RequestRectForFoundTextRangeAsyncReply = CompletionHandler<void(const WebCore::FloatRect& rect)>;

using AddLayerForFindOverlayAsyncReply = CompletionHandler<void(const WebCore::GraphicsLayer::PlatformLayerID& findLayerID)>;

using RemoveLayerForFindOverlayAsyncReply = CompletionHandler<void()>;

#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
using InsertDroppedImagePlaceholdersAsyncReply = CompletionHandler<void(const Vector<WebCore::IntRect>& dropPreviewRects, const std::optional<WebCore::TextIndicatorData>& textIndicator)>;
#endif

#if ENABLE(MEDIA_STREAM)
using UserMediaAccessWasGrantedAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(ENCRYPTED_MEDIA)
using MediaKeySystemWasGrantedAsyncReply = CompletionHandler<void()>;
#endif

using RequestMediaPlaybackStateAsyncReply = CompletionHandler<void(WebKit::MediaPlaybackState state)>;

using PauseAllMediaPlaybackAsyncReply = CompletionHandler<void()>;

using SuspendAllMediaPlaybackAsyncReply = CompletionHandler<void()>;

using ResumeAllMediaPlaybackAsyncReply = CompletionHandler<void()>;

using IsLayerTreeFrozenAsyncReply = CompletionHandler<void(bool isFrozen)>;

using ComputePagesForPrintingAsyncReply = CompletionHandler<void(const Vector<WebCore::IntRect>& pageRects, double totalScaleFactorForPrinting, const WebCore::RectEdges<float>& computedPageMargin)>;

#if PLATFORM(COCOA)
using DrawRectToImageAsyncReply = CompletionHandler<void(const WebKit::ShareableBitmap::Handle& image)>;
#endif

#if PLATFORM(COCOA)
using DrawPagesToPDFAsyncReply = CompletionHandler<void(const RefPtr<WebCore::SharedBuffer>& data)>;
#endif

#if (PLATFORM(COCOA) && PLATFORM(IOS_FAMILY))
using ComputePagesForPrintingiOSDelayedReply = CompletionHandler<void(size_t pageCount)>;
#endif

#if (PLATFORM(COCOA) && PLATFORM(IOS_FAMILY))
using DrawToPDFiOSAsyncReply = CompletionHandler<void(const RefPtr<WebCore::SharedBuffer>& data)>;
#endif

#if PLATFORM(COCOA)
using DrawToPDFAsyncReply = CompletionHandler<void(const RefPtr<WebCore::SharedBuffer>& data)>;
#endif

#if PLATFORM(GTK)
using DrawPagesForPrintingAsyncReply = CompletionHandler<void(const std::optional<WebKit::SharedMemory::Handle>& data, const WebCore::ResourceError& error)>;
#endif

using SetMutedAsyncReply = CompletionHandler<void()>;

using StopMediaCaptureAsyncReply = CompletionHandler<void()>;

#if PLATFORM(COCOA)
using GetStringSelectionForPasteboardDelayedReply = CompletionHandler<void(const String& stringValue)>;
#endif

#if PLATFORM(COCOA)
using GetDataSelectionForPasteboardDelayedReply = CompletionHandler<void(const RefPtr<WebCore::SharedBuffer>& buffer)>;
#endif

#if PLATFORM(COCOA)
using ReadSelectionFromPasteboardDelayedReply = CompletionHandler<void(bool result)>;
#endif

#if PLATFORM(COCOA)
using ShouldDelayWindowOrderingEventDelayedReply = CompletionHandler<void(bool result)>;
#endif

#if PLATFORM(COCOA)
using AddDictationAlternativeAsyncReply = CompletionHandler<void(bool success)>;
#endif

#if PLATFORM(COCOA)
using DictationAlternativesAtSelectionAsyncReply = CompletionHandler<void(const Vector<WebCore::DictationContext>& contexts)>;
#endif

#if PLATFORM(COCOA)
using HasMarkedTextAsyncReply = CompletionHandler<void(bool hasMarkedText)>;
#endif

#if PLATFORM(COCOA)
using GetMarkedRangeAsyncAsyncReply = CompletionHandler<void(const WebKit::EditingRange& range)>;
#endif

#if PLATFORM(COCOA)
using GetSelectedRangeAsyncAsyncReply = CompletionHandler<void(const WebKit::EditingRange& range)>;
#endif

#if PLATFORM(COCOA)
using CharacterIndexForPointAsyncAsyncReply = CompletionHandler<void(uint64_t location)>;
#endif

#if PLATFORM(COCOA)
using FirstRectForCharacterRangeAsyncAsyncReply = CompletionHandler<void(const WebCore::IntRect& rect, const WebKit::EditingRange& actualRange)>;
#endif

#if PLATFORM(MAC)
using AttributedSubstringForCharacterRangeAsyncAsyncReply = CompletionHandler<void(const WebCore::AttributedString& string, const WebKit::EditingRange& range)>;
#endif

#if ENABLE(IMAGE_ANALYSIS_ENHANCEMENTS)
using ShouldAllowRemoveBackgroundAsyncReply = CompletionHandler<void(bool result)>;
#endif

using GetBytecodeProfileAsyncReply = CompletionHandler<void(const String& string)>;

using GetSamplingProfilerOutputAsyncReply = CompletionHandler<void(const String& string)>;

using TakeSnapshotAsyncReply = CompletionHandler<void(const WebKit::ShareableBitmap::Handle& image)>;

#if PLATFORM(COCOA)
using RequestActiveNowPlayingSessionInfoAsyncReply = CompletionHandler<void(bool active, bool registeredAsNowPlayingApplication, const String& title, double duration, double elapsedTime, uint64_t uniqueIdentifier)>;
#endif

using DidGetLoadDecisionForIconAsyncReply = CompletionHandler<void(const IPC::SharedBufferReference& iconData)>;

using URLSchemeTaskWillPerformRedirectionAsyncReply = CompletionHandler<void(const WebCore::ResourceRequest& actualRequest)>;

#if ENABLE(ATTACHMENT_ELEMENT)
using InsertAttachmentAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
using UpdateAttachmentAttributesAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(APPLICATION_MANIFEST)
using GetApplicationManifestAsyncReply = CompletionHandler<void(const std::optional<WebCore::ApplicationManifest>& manifest)>;
#endif

#if ENABLE(UI_PROCESS_PDF_HUD)
using SavePDFAsyncReply = CompletionHandler<void(const String& filename, const URL& url, const IPC::DataReference& data)>;
#endif

#if ENABLE(UI_PROCESS_PDF_HUD)
using OpenPDFWithPreviewAsyncReply = CompletionHandler<void(const String& filename, const WebKit::FrameInfoData& frameInfo, const IPC::DataReference& data, const String& uuid)>;
#endif

#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
using GetLoadedSubresourceDomainsAsyncReply = CompletionHandler<void(const Vector<WebCore::RegistrableDomain>& domains)>;
#endif

#if PLATFORM(GTK) || PLATFORM(WPE)
using SendMessageToWebExtensionWithReplyAsyncReply = CompletionHandler<void(const WebKit::UserMessage& replyMessage)>;
#endif

using StartTextManipulationsAsyncReply = CompletionHandler<void()>;

using CompleteTextManipulationAsyncReply = CompletionHandler<void(bool allFailed, const Vector<WebCore::TextManipulationController::ManipulationFailure>& failures)>;

using GetProcessDisplayNameAsyncReply = CompletionHandler<void(const String& displayName)>;

using DispatchWheelEventWithoutScrollingAsyncReply = CompletionHandler<void(bool handled)>;

using LastNavigationWasAppInitiatedAsyncReply = CompletionHandler<void(bool wasAppBound)>;

#if ENABLE(MEDIA_SESSION_COORDINATOR)
using CreateMediaSessionCoordinatorAsyncReply = CompletionHandler<void(bool success)>;
#endif

#if ENABLE(IMAGE_ANALYSIS)
using UpdateWithTextRecognitionResultAsyncReply = CompletionHandler<void(WebKit::TextRecognitionUpdateResult result)>;
#endif

using NavigateServiceWorkerClientAsyncReply = CompletionHandler<void(bool result)>;

} // namespace WebPage
} // namespace Messages
