/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "MessageNames.h"
#include "SandboxExtension.h"
#include <WebCore/PasteboardItemInfo.h>
#include <WebCore/SharedBuffer.h>
#include <optional>
#include <wtf/Forward.h>
#include <wtf/RefCounted.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebCore {
class Color;
}

namespace Messages {
namespace WebPasteboardProxy {

using WriteCustomDataDelayedReply = CompletionHandler<void(int64_t changeCount)>;

using TypesSafeForDOMToReadAndWriteDelayedReply = CompletionHandler<void(const Vector<String>& types)>;

using AllPasteboardItemInfoDelayedReply = CompletionHandler<void(const std::optional<Vector<WebCore::PasteboardItemInfo>>& allInfo)>;

using InformationForItemAtIndexDelayedReply = CompletionHandler<void(const std::optional<WebCore::PasteboardItemInfo>& info)>;

using GetPasteboardItemsCountDelayedReply = CompletionHandler<void(uint64_t itemsCount)>;

using ReadStringFromPasteboardDelayedReply = CompletionHandler<void(const String& string)>;

using ReadURLFromPasteboardDelayedReply = CompletionHandler<void(const String& url, const String& title)>;

using ReadBufferFromPasteboardDelayedReply = CompletionHandler<void(const RefPtr<WebCore::SharedBuffer>& buffer)>;

using ContainsStringSafeForDOMToReadForTypeDelayedReply = CompletionHandler<void(bool result)>;

#if PLATFORM(COCOA)
using GetNumberOfFilesDelayedReply = CompletionHandler<void(uint64_t numberOfFiles)>;
#endif

#if PLATFORM(COCOA)
using GetPasteboardTypesDelayedReply = CompletionHandler<void(const Vector<String>& types)>;
#endif

#if PLATFORM(COCOA)
using GetPasteboardPathnamesForTypeDelayedReply = CompletionHandler<void(const Vector<String>& pathnames, const Vector<WebKit::SandboxExtension::Handle>& sandboxExtensions)>;
#endif

#if PLATFORM(COCOA)
using GetPasteboardStringForTypeDelayedReply = CompletionHandler<void(const String& string)>;
#endif

#if PLATFORM(COCOA)
using GetPasteboardStringsForTypeDelayedReply = CompletionHandler<void(const Vector<String>& strings)>;
#endif

#if PLATFORM(COCOA)
using GetPasteboardBufferForTypeDelayedReply = CompletionHandler<void(const RefPtr<WebCore::SharedBuffer>& buffer)>;
#endif

#if PLATFORM(COCOA)
using GetPasteboardChangeCountDelayedReply = CompletionHandler<void(int64_t changeCount)>;
#endif

#if PLATFORM(COCOA)
using GetPasteboardColorDelayedReply = CompletionHandler<void(const WebCore::Color& color)>;
#endif

#if PLATFORM(COCOA)
using GetPasteboardURLDelayedReply = CompletionHandler<void(const String& urlString)>;
#endif

#if PLATFORM(COCOA)
using AddPasteboardTypesDelayedReply = CompletionHandler<void(int64_t changeCount)>;
#endif

#if PLATFORM(COCOA)
using SetPasteboardTypesDelayedReply = CompletionHandler<void(int64_t changeCount)>;
#endif

#if PLATFORM(COCOA)
using SetPasteboardURLDelayedReply = CompletionHandler<void(int64_t changeCount)>;
#endif

#if PLATFORM(COCOA)
using SetPasteboardColorDelayedReply = CompletionHandler<void(int64_t changeCount)>;
#endif

#if PLATFORM(COCOA)
using SetPasteboardStringForTypeDelayedReply = CompletionHandler<void(int64_t changeCount)>;
#endif

#if PLATFORM(COCOA)
using SetPasteboardBufferForTypeDelayedReply = CompletionHandler<void(int64_t changeCount)>;
#endif

#if PLATFORM(COCOA)
using ContainsURLStringSuitableForLoadingDelayedReply = CompletionHandler<void(bool result)>;
#endif

#if PLATFORM(COCOA)
using URLStringSuitableForLoadingDelayedReply = CompletionHandler<void(const String& url, const String& title)>;
#endif

#if (PLATFORM(COCOA) && ENABLE(IPC_TESTING_API))
using TestIPCSharedMemoryDelayedReply = CompletionHandler<void(int64_t changeCount, const String& buffer)>;
#endif

#if PLATFORM(GTK)
using GetTypesDelayedReply = CompletionHandler<void(const Vector<String>& types)>;
#endif

#if PLATFORM(GTK)
using ReadTextDelayedReply = CompletionHandler<void(const String& text)>;
#endif

#if PLATFORM(GTK)
using ReadFilePathsDelayedReply = CompletionHandler<void(const Vector<String>& types)>;
#endif

#if PLATFORM(GTK)
using ReadBufferDelayedReply = CompletionHandler<void(const RefPtr<WebCore::SharedBuffer>& buffer)>;
#endif

#if USE(LIBWPE)
using GetPasteboardTypesDelayedReply = CompletionHandler<void(const Vector<String>& types)>;
#endif

} // namespace WebPasteboardProxy
} // namespace Messages
