/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if ENABLE(SERVICE_WORKER)

#include "MessageNames.h"
#include <WebCore/ExceptionData.h>
#include <WebCore/NavigationPreloadState.h>
#include <WebCore/PushSubscriptionData.h>
#include <WebCore/ServiceWorkerRegistrationData.h>
#include <optional>
#include <wtf/Expected.h>
#include <wtf/Forward.h>
#include <wtf/Vector.h>


namespace Messages {
namespace WebSWServerConnection {

using ScheduleUnregisterJobInServerAsyncReply = CompletionHandler<void(const Expected<bool, WebCore::ExceptionData>& result)>;

using MatchRegistrationAsyncReply = CompletionHandler<void(const std::optional<WebCore::ServiceWorkerRegistrationData>& registration)>;

using WhenRegistrationReadyAsyncReply = CompletionHandler<void(const std::optional<WebCore::ServiceWorkerRegistrationData>& registration)>;

using GetRegistrationsAsyncReply = CompletionHandler<void(const Vector<WebCore::ServiceWorkerRegistrationData>& registrations)>;

using TerminateWorkerFromClientAsyncReply = CompletionHandler<void()>;

using WhenServiceWorkerIsTerminatedForTestingAsyncReply = CompletionHandler<void()>;

using StoreRegistrationsOnDiskAsyncReply = CompletionHandler<void()>;

using SubscribeToPushServiceAsyncReply = CompletionHandler<void(const Expected<WebCore::PushSubscriptionData, WebCore::ExceptionData>& result)>;

using UnsubscribeFromPushServiceAsyncReply = CompletionHandler<void(const Expected<bool, WebCore::ExceptionData>& result)>;

using GetPushSubscriptionAsyncReply = CompletionHandler<void(const Expected<std::optional<WebCore::PushSubscriptionData>, WebCore::ExceptionData>& result)>;

using GetPushPermissionStateAsyncReply = CompletionHandler<void(const Expected<uint8_t, WebCore::ExceptionData>& result)>;

using EnableNavigationPreloadAsyncReply = CompletionHandler<void(const std::optional<WebCore::ExceptionData>& result)>;

using DisableNavigationPreloadAsyncReply = CompletionHandler<void(const std::optional<WebCore::ExceptionData>& result)>;

using SetNavigationPreloadHeaderValueAsyncReply = CompletionHandler<void(const std::optional<WebCore::ExceptionData>& result)>;

using GetNavigationPreloadStateAsyncReply = CompletionHandler<void(const Expected<WebCore::NavigationPreloadState, WebCore::ExceptionData>& result)>;

} // namespace WebSWServerConnection
} // namespace Messages

#endif // ENABLE(SERVICE_WORKER)
